@@ -7,10 +7,12 @@ static const char zHelp[] =
   "Usage: %s [--options] DATABASE\n"
   "Options:\n"
   "  --autovacuum        Enable AUTOVACUUM mode\n"
-  "  --cachesize N       Set the cache size to N\n"
+  "  --big-transactions  Add BEGIN/END around all large tests\n"
+  "  --cachesize N       Set PRAGMA cache_size=N. Note: N is pages, not bytes\n"
   "  --checkpoint        Run PRAGMA wal_checkpoint after each test case\n"
   "  --exclusive         Enable locking_mode=EXCLUSIVE\n"
   "  --explain           Like --sqlonly but with added EXPLAIN keywords\n"
+  "  --fullfsync         Enable fullfsync=TRUE\n"
   "  --heap SZ MIN       Memory allocator uses SZ bytes & min allocation MIN\n"
   "  --incrvacuum        Enable incremenatal vacuum mode\n"
   "  --journal M         Set the journal_mode to M\n"
@@ -20,6 +22,7 @@ static const char zHelp[] =
   "  --mmap SZ           MMAP the first SZ bytes of the database file\n"
   "  --multithread       Set multithreaded mode\n"
   "  --nomemstat         Disable memory statistics\n"
+  "  --nomutex           Open db with SQLITE_OPEN_NOMUTEX\n"
   "  --nosync            Set PRAGMA synchronous=OFF\n"
   "  --notnull           Add NOT NULL constraints to table columns\n"
   "  --output FILE       Store SQL output in FILE\n"
@@ -29,19 +32,23 @@ static const char zHelp[] =
   "  --repeat N          Repeat each SELECT N times (default: 1)\n"
   "  --reprepare         Reprepare each statement upon every invocation\n"
   "  --reserve N         Reserve N bytes on each database page\n"
+  "  --script FILE       Write an SQL script for the test into FILE\n"
   "  --serialized        Set serialized threading mode\n"
   "  --singlethread      Set single-threaded mode - disables all mutexing\n"
   "  --sqlonly           No-op.  Only show the SQL that would have been run.\n"
   "  --shrink-memory     Invoke sqlite3_db_release_memory() frequently.\n"
   "  --size N            Relative test size.  Default=100\n"
+  "  --strict            Use STRICT table where appropriate\n"
   "  --stats             Show statistics at the end\n"
+  "  --stmtscanstatus    Activate SQLITE_DBCONFIG_STMT_SCANSTATUS\n"
   "  --temp N            N from 0 to 9.  0: no temp table. 9: all temp tables\n"
   "  --testset T         Run test-set T (main, cte, rtree, orm, fp, debug)\n"
   "  --trace             Turn on SQL tracing\n"
   "  --threads N         Use up to N threads for sorting\n"
   "  --utf16be           Set text encoding to UTF-16BE\n"
   "  --utf16le           Set text encoding to UTF-16LE\n"
-  "  --verify            Run additional verification steps.\n"
+  "  --verify            Run additional verification steps\n"
+  "  --vfs NAME          Use the given (preinstalled) VFS\n"
   "  --without-rowid     Use WITHOUT ROWID where appropriate\n"
 ;
 
@@ -95,6 +102,8 @@ static struct Global {
   int nRepeat;               /* Repeat selects this many times */
   int doCheckpoint;          /* Run PRAGMA wal_checkpoint after each trans */
   int nReserve;              /* Reserve bytes */
+  int stmtScanStatus;        /* True to activate Stmt ScanStatus reporting */
+  int doBigTransactions;     /* Enable transactions on tests 410 and 510 */
   const char *zWR;           /* Might be WITHOUT ROWID */
   const char *zNN;           /* Might be NOT NULL */
   const char *zPK;           /* Might be UNIQUE or PRIMARY KEY */
@@ -102,6 +111,7 @@ static struct Global {
   u64 nResByte;              /* Total number of result bytes */
   int nResult;               /* Size of the current result */
   char zResult[3000];        /* Text of the current result */
+  FILE *pScript;             /* Write an SQL script into this file */
 #ifndef SPEEDTEST_OMIT_HASH
   FILE *hashFile;            /* Store all hash results in this file */
   HashContext hash;          /* Hash of all output */
@@ -369,10 +379,12 @@ int speedtest1_numbername(unsigned int n, char *zOut, int nOut){
 #define NAMEWIDTH 60
 static const char zDots[] =
   ".......................................................................";
+static int iTestNumber = 0;  /* Current test # for begin/end_test(). */
 void speedtest1_begin_test(int iTestNum, const char *zTestName, ...){
   int n = (int)strlen(zTestName);
   char *zName;
   va_list ap;
+  iTestNumber = iTestNum;
   va_start(ap, zTestName);
   zName = sqlite3_vmprintf(zTestName, ap);
   va_end(ap);
@@ -381,6 +393,11 @@ void speedtest1_begin_test(int iTestNum, const char *zTestName, ...){
     zName[NAMEWIDTH] = 0;
     n = NAMEWIDTH;
   }
+  if( g.pScript ){
+    fprintf(g.pScript,"-- begin test %d %.*s\n", iTestNumber, n, zName)
+      /* maintenance reminder: ^^^ code in ext/wasm expects %d to be
+      ** field #4 (as in: cut -d' ' -f4). */;
+  }
   if( g.bSqlOnly ){
     printf("/* %4d - %s%.*s */\n", iTestNum, zName, NAMEWIDTH-n, zDots);
   }else{
@@ -401,6 +418,10 @@ void speedtest1_exec(const char*,...);
 void speedtest1_end_test(void){
   sqlite3_int64 iElapseTime = speedtest1_timestamp() - g.iStart;
   if( g.doCheckpoint ) speedtest1_exec("PRAGMA wal_checkpoint;");
+  assert( iTestNumber > 0 );
+  if( g.pScript ){
+    fprintf(g.pScript,"-- end test %d\n", iTestNumber);
+  }
   if( !g.bSqlOnly ){
     g.iTotal += iElapseTime;
     printf("%4d.%03ds\n", (int)(iElapseTime/1000), (int)(iElapseTime%1000));
@@ -409,6 +430,7 @@ void speedtest1_end_test(void){
     sqlite3_finalize(g.pStmt);
     g.pStmt = 0;
   }
+  iTestNumber = 0;
 }
 
 /* Report end of testing */
@@ -472,7 +494,11 @@ void speedtest1_exec(const char *zFormat, ...){
     printSql(zSql);
   }else{
     char *zErrMsg = 0;
-    int rc = sqlite3_exec(g.db, zSql, 0, 0, &zErrMsg);
+    int rc;
+    if( g.pScript ){
+      fprintf(g.pScript,"%s;\n",zSql);
+    }
+    rc = sqlite3_exec(g.db, zSql, 0, 0, &zErrMsg);
     if( zErrMsg ) fatal_error("SQL error: %s\n%s\n", zErrMsg, zSql);
     if( rc!=SQLITE_OK ) fatal_error("exec error: %s\n", sqlite3_errmsg(g.db));
   }
@@ -499,6 +525,11 @@ char *speedtest1_once(const char *zFormat, ...){
     if( rc ){
       fatal_error("SQL error: %s\n", sqlite3_errmsg(g.db));
     }
+    if( g.pScript ){
+      char *z = sqlite3_expanded_sql(pStmt);
+      fprintf(g.pScript,"%s\n",z);
+      sqlite3_free(z);
+    }
     if( sqlite3_step(pStmt)==SQLITE_ROW ){
       const char *z = (const char*)sqlite3_column_text(pStmt, 0);
       if( z ) zResult = sqlite3_mprintf("%s", z);
@@ -536,6 +567,11 @@ void speedtest1_run(void){
   if( g.bSqlOnly ) return;
   assert( g.pStmt );
   g.nResult = 0;
+  if( g.pScript ){
+    char *z = sqlite3_expanded_sql(g.pStmt);
+    fprintf(g.pScript,"%s\n",z);
+    sqlite3_free(z);
+  }
   while( sqlite3_step(g.pStmt)==SQLITE_ROW ){
     n = sqlite3_column_count(g.pStmt);
     for(i=0; i<n; i++){
@@ -707,9 +743,9 @@ void testset_main(void){
   maxb = roundup_allones(sz);
   speedtest1_begin_test(100, "%d INSERTs into table with no index", n);
   speedtest1_exec("BEGIN");
-  speedtest1_exec("CREATE%s TABLE t1(a INTEGER %s, b INTEGER %s, c TEXT %s);",
+  speedtest1_exec("CREATE%s TABLE z1(a INTEGER %s, b INTEGER %s, c TEXT %s);",
                   isTemp(9), g.zNN, g.zNN, g.zNN);
-  speedtest1_prepare("INSERT INTO t1 VALUES(?1,?2,?3); --  %d times", n);
+  speedtest1_prepare("INSERT INTO z1 VALUES(?1,?2,?3); --  %d times", n);
   for(i=1; i<=n; i++){
     x1 = swizzle(i,maxb);
     speedtest1_numbername(x1, zNum, sizeof(zNum));
@@ -726,9 +762,9 @@ void testset_main(void){
   speedtest1_begin_test(110, "%d ordered INSERTS with one index/PK", n);
   speedtest1_exec("BEGIN");
   speedtest1_exec(
-     "CREATE%s TABLE t2(a INTEGER %s %s, b INTEGER %s, c TEXT %s) %s",
+     "CREATE%s TABLE z2(a INTEGER %s %s, b INTEGER %s, c TEXT %s) %s",
      isTemp(5), g.zNN, g.zPK, g.zNN, g.zNN, g.zWR);
-  speedtest1_prepare("INSERT INTO t2 VALUES(?1,?2,?3); -- %d times", n);
+  speedtest1_prepare("INSERT INTO z2 VALUES(?1,?2,?3); -- %d times", n);
   for(i=1; i<=n; i++){
     x1 = swizzle(i,maxb);
     speedtest1_numbername(x1, zNum, sizeof(zNum));
@@ -768,7 +804,7 @@ void testset_main(void){
   speedtest1_begin_test(130, "%d SELECTS, numeric BETWEEN, unindexed", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "SELECT count(*), avg(b), sum(length(c)), group_concat(c) FROM t1\n"
+    "SELECT count(*), avg(b), sum(length(c)), group_concat(c) FROM z1\n"
     " WHERE b BETWEEN ?1 AND ?2; -- %d times", n
   );
   for(i=1; i<=n; i++){
@@ -788,7 +824,7 @@ void testset_main(void){
   speedtest1_begin_test(140, "%d SELECTS, LIKE, unindexed", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "SELECT count(*), avg(b), sum(length(c)), group_concat(c) FROM t1\n"
+    "SELECT count(*), avg(b), sum(length(c)), group_concat(c) FROM z1\n"
     " WHERE c LIKE ?1; -- %d times", n
   );
   for(i=1; i<=n; i++){
@@ -810,7 +846,7 @@ void testset_main(void){
   speedtest1_begin_test(142, "%d SELECTS w/ORDER BY, unindexed", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "SELECT a, b, c FROM t1 WHERE c LIKE ?1\n"
+    "SELECT a, b, c FROM z1 WHERE c LIKE ?1\n"
     " ORDER BY a; -- %d times", n
   );
   for(i=1; i<=n; i++){
@@ -831,7 +867,7 @@ void testset_main(void){
   speedtest1_begin_test(145, "%d SELECTS w/ORDER BY and LIMIT, unindexed", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "SELECT a, b, c FROM t1 WHERE c LIKE ?1\n"
+    "SELECT a, b, c FROM z1 WHERE c LIKE ?1\n"
     " ORDER BY a LIMIT 10; -- %d times", n
   );
   for(i=1; i<=n; i++){
@@ -851,10 +887,10 @@ void testset_main(void){
 
   speedtest1_begin_test(150, "CREATE INDEX five times");
   speedtest1_exec("BEGIN;");
-  speedtest1_exec("CREATE UNIQUE INDEX t1b ON t1(b);");
-  speedtest1_exec("CREATE INDEX t1c ON t1(c);");
-  speedtest1_exec("CREATE UNIQUE INDEX t2b ON t2(b);");
-  speedtest1_exec("CREATE INDEX t2c ON t2(c DESC);");
+  speedtest1_exec("CREATE UNIQUE INDEX t1b ON z1(b);");
+  speedtest1_exec("CREATE INDEX t1c ON z1(c);");
+  speedtest1_exec("CREATE UNIQUE INDEX t2b ON z2(b);");
+  speedtest1_exec("CREATE INDEX t2c ON z2(c DESC);");
   speedtest1_exec("CREATE INDEX t3bc ON t3(b,c);");
   speedtest1_exec("COMMIT;");
   speedtest1_end_test();
@@ -864,7 +900,7 @@ void testset_main(void){
   speedtest1_begin_test(160, "%d SELECTS, numeric BETWEEN, indexed", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "SELECT count(*), avg(b), sum(length(c)), group_concat(a) FROM t1\n"
+    "SELECT count(*), avg(b), sum(length(c)), group_concat(a) FROM z1\n"
     " WHERE b BETWEEN ?1 AND ?2; -- %d times", n
   );
   for(i=1; i<=n; i++){
@@ -884,7 +920,7 @@ void testset_main(void){
   speedtest1_begin_test(161, "%d SELECTS, numeric BETWEEN, PK", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "SELECT count(*), avg(b), sum(length(c)), group_concat(a) FROM t2\n"
+    "SELECT count(*), avg(b), sum(length(c)), group_concat(a) FROM z2\n"
     " WHERE a BETWEEN ?1 AND ?2; -- %d times", n
   );
   for(i=1; i<=n; i++){
@@ -904,7 +940,7 @@ void testset_main(void){
   speedtest1_begin_test(170, "%d SELECTS, text BETWEEN, indexed", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "SELECT count(*), avg(b), sum(length(c)), group_concat(a) FROM t1\n"
+    "SELECT count(*), avg(b), sum(length(c)), group_concat(a) FROM z1\n"
     " WHERE c BETWEEN ?1 AND (?1||'~'); -- %d times", n
   );
   for(i=1; i<=n; i++){
@@ -930,14 +966,14 @@ void testset_main(void){
     isTemp(1), g.zNN, g.zPK, g.zNN, g.zNN, g.zWR);
   speedtest1_exec("CREATE INDEX t4b ON t4(b)");
   speedtest1_exec("CREATE INDEX t4c ON t4(c)");
-  speedtest1_exec("INSERT INTO t4 SELECT * FROM t1");
+  speedtest1_exec("INSERT INTO t4 SELECT * FROM z1");
   speedtest1_exec("COMMIT");
   speedtest1_end_test();
 
   n = sz;
   speedtest1_begin_test(190, "DELETE and REFILL one table", n);
-  speedtest1_exec("DELETE FROM t2;");
-  speedtest1_exec("INSERT INTO t2 SELECT * FROM t1;");
+  speedtest1_exec("DELETE FROM z2;");
+  speedtest1_exec("INSERT INTO z2 SELECT * FROM z1;");
   speedtest1_end_test();
 
 
@@ -947,8 +983,8 @@ void testset_main(void){
 
 
   speedtest1_begin_test(210, "ALTER TABLE ADD COLUMN, and query");
-  speedtest1_exec("ALTER TABLE t2 ADD COLUMN d DEFAULT 123");
-  speedtest1_exec("SELECT sum(d) FROM t2");
+  speedtest1_exec("ALTER TABLE z2 ADD COLUMN d INT DEFAULT 123");
+  speedtest1_exec("SELECT sum(d) FROM z2");
   speedtest1_end_test();
 
 
@@ -956,7 +992,7 @@ void testset_main(void){
   speedtest1_begin_test(230, "%d UPDATES, numeric BETWEEN, indexed", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "UPDATE t2 SET d=b*2 WHERE b BETWEEN ?1 AND ?2; -- %d times", n
+    "UPDATE z2 SET d=b*2 WHERE b BETWEEN ?1 AND ?2; -- %d times", n
   );
   for(i=1; i<=n; i++){
     x1 = speedtest1_random()%maxb;
@@ -973,7 +1009,7 @@ void testset_main(void){
   speedtest1_begin_test(240, "%d UPDATES of individual rows", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "UPDATE t2 SET d=b*3 WHERE a=?1; -- %d times", n
+    "UPDATE z2 SET d=b*3 WHERE a=?1; -- %d times", n
   );
   for(i=1; i<=n; i++){
     x1 = speedtest1_random()%sz + 1;
@@ -984,12 +1020,12 @@ void testset_main(void){
   speedtest1_end_test();
 
   speedtest1_begin_test(250, "One big UPDATE of the whole %d-row table", sz);
-  speedtest1_exec("UPDATE t2 SET d=b*4");
+  speedtest1_exec("UPDATE z2 SET d=b*4");
   speedtest1_end_test();
 
 
   speedtest1_begin_test(260, "Query added column after filling");
-  speedtest1_exec("SELECT sum(d) FROM t2");
+  speedtest1_exec("SELECT sum(d) FROM z2");
   speedtest1_end_test();
 
 
@@ -998,7 +1034,7 @@ void testset_main(void){
   speedtest1_begin_test(270, "%d DELETEs, numeric BETWEEN, indexed", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "DELETE FROM t2 WHERE b BETWEEN ?1 AND ?2; -- %d times", n
+    "DELETE FROM z2 WHERE b BETWEEN ?1 AND ?2; -- %d times", n
   );
   for(i=1; i<=n; i++){
     x1 = speedtest1_random()%maxb + 1;
@@ -1027,16 +1063,16 @@ void testset_main(void){
 
 
   speedtest1_begin_test(290, "Refill two %d-row tables using REPLACE", sz);
-  speedtest1_exec("REPLACE INTO t2(a,b,c) SELECT a,b,c FROM t1");
-  speedtest1_exec("REPLACE INTO t3(a,b,c) SELECT a,b,c FROM t1");
+  speedtest1_exec("REPLACE INTO z2(a,b,c) SELECT a,b,c FROM z1");
+  speedtest1_exec("REPLACE INTO t3(a,b,c) SELECT a,b,c FROM z1");
   speedtest1_end_test();
 
   speedtest1_begin_test(300, "Refill a %d-row table using (b&1)==(a&1)", sz);
-  speedtest1_exec("DELETE FROM t2;");
-  speedtest1_exec("INSERT INTO t2(a,b,c)\n"
-                  " SELECT a,b,c FROM t1  WHERE (b&1)==(a&1);");
-  speedtest1_exec("INSERT INTO t2(a,b,c)\n"
-                  " SELECT a,b,c FROM t1  WHERE (b&1)<>(a&1);");
+  speedtest1_exec("DELETE FROM z2;");
+  speedtest1_exec("INSERT INTO z2(a,b,c)\n"
+                  " SELECT a,b,c FROM z1  WHERE (b&1)==(a&1);");
+  speedtest1_exec("INSERT INTO z2(a,b,c)\n"
+                  " SELECT a,b,c FROM z1  WHERE (b&1)<>(a&1);");
   speedtest1_end_test();
 
 
@@ -1044,11 +1080,11 @@ void testset_main(void){
   speedtest1_begin_test(310, "%d four-ways joins", n);
   speedtest1_exec("BEGIN");
   speedtest1_prepare(
-    "SELECT t1.c FROM t1, t2, t3, t4\n"
+    "SELECT z1.c FROM z1, z2, t3, t4\n"
     " WHERE t4.a BETWEEN ?1 AND ?2\n"
     "   AND t3.a=t4.b\n"
-    "   AND t2.a=t3.b\n"
-    "   AND t1.c=t2.c"
+    "   AND z2.a=t3.b\n"
+    "   AND z1.c=z2.c;"
   );
   for(i=1; i<=n; i++){
     x1 = speedtest1_random()%sz + 1;
@@ -1063,8 +1099,8 @@ void testset_main(void){
   speedtest1_begin_test(320, "subquery in result set", n);
   speedtest1_prepare(
     "SELECT sum(a), max(c),\n"
-    "       avg((SELECT a FROM t2 WHERE 5+t2.b=t1.b) AND rowid<?1), max(c)\n"
-    " FROM t1 WHERE rowid<?1;"
+    "       avg((SELECT a FROM z2 WHERE 5+z2.b=z1.b) AND rowid<?1), max(c)\n"
+    " FROM z1 WHERE rowid<?1;"
   );
   sqlite3_bind_int(g.pStmt, 1, est_square_root(g.szTest)*50);
   speedtest1_run();
@@ -1088,12 +1124,24 @@ void testset_main(void){
   speedtest1_exec("COMMIT");
   speedtest1_end_test();
   speedtest1_begin_test(410, "%d SELECTS on an IPK", n);
+  if( g.doBigTransactions ){
+    /* Historical note: tests 410 and 510 have historically not used
+    ** explicit transactions. The --big-transactions flag was added
+    ** 2022-09-08 to support the WASM/OPFS build, as the run-times
+    ** approach 1 minute for each of these tests if they're not in an
+    ** explicit transaction. The run-time effect of --big-transaciions
+    ** on native builds is negligible. */
+    speedtest1_exec("BEGIN");
+  }
   speedtest1_prepare("SELECT b FROM t5 WHERE a=?1; --  %d times",n);
   for(i=1; i<=n; i++){
     x1 = swizzle(i,maxb);
     sqlite3_bind_int(g.pStmt, 1, (sqlite3_int64)x1);
     speedtest1_run();
   }
+  if( g.doBigTransactions ){
+    speedtest1_exec("COMMIT");
+  }
   speedtest1_end_test();
 
   sz = n = g.szTest*700;
@@ -1115,6 +1163,10 @@ void testset_main(void){
   speedtest1_exec("COMMIT");
   speedtest1_end_test();
   speedtest1_begin_test(510, "%d SELECTS on a TEXT PK", n);
+  if( g.doBigTransactions ){
+    /* See notes for test 410. */
+    speedtest1_exec("BEGIN");
+  }
   speedtest1_prepare("SELECT b FROM t6 WHERE a=?1; --  %d times",n);
   for(i=1; i<=n; i++){
     x1 = swizzle(i,maxb);
@@ -1122,6 +1174,9 @@ void testset_main(void){
     sqlite3_bind_text(g.pStmt, 1, zNum, -1, SQLITE_STATIC);
     speedtest1_run();
   }
+  if( g.doBigTransactions ){
+    speedtest1_exec("COMMIT");
+  }
   speedtest1_end_test();
   speedtest1_begin_test(520, "%d SELECT DISTINCT", n);
   speedtest1_exec("SELECT DISTINCT b FROM t5;");
@@ -1282,10 +1337,10 @@ void testset_cte(void){
   speedtest1_begin_test(400, "EXCEPT operator on %d-element tables", nElem);
   speedtest1_prepare(
     "WITH RECURSIVE \n"
-    "  t1(x) AS (VALUES(2) UNION ALL SELECT x+2 FROM t1 WHERE x<%d),\n"
-    "  t2(y) AS (VALUES(3) UNION ALL SELECT y+3 FROM t2 WHERE y<%d)\n"
+    "  z1(x) AS (VALUES(2) UNION ALL SELECT x+2 FROM z1 WHERE x<%d),\n"
+    "  z2(y) AS (VALUES(3) UNION ALL SELECT y+3 FROM z2 WHERE y<%d)\n"
     "SELECT count(x), avg(x) FROM (\n"
-    "  SELECT x FROM t1 EXCEPT SELECT y FROM t2 ORDER BY 1\n"
+    "  SELECT x FROM z1 EXCEPT SELECT y FROM z2 ORDER BY 1\n"
     ");",
     nElem, nElem
   );
@@ -1318,9 +1373,9 @@ void testset_fp(void){
   n = g.szTest*5000;
   speedtest1_begin_test(100, "Fill a table with %d FP values", n*2);
   speedtest1_exec("BEGIN");
-  speedtest1_exec("CREATE%s TABLE t1(a REAL %s, b REAL %s);",
+  speedtest1_exec("CREATE%s TABLE z1(a REAL %s, b REAL %s);",
                   isTemp(1), g.zNN, g.zNN);
-  speedtest1_prepare("INSERT INTO t1 VALUES(?1,?2); -- %d times", n);
+  speedtest1_prepare("INSERT INTO z1 VALUES(?1,?2); -- %d times", n);
   for(i=1; i<=n; i++){
     speedtest1_random_ascii_fp(zFP1);
     speedtest1_random_ascii_fp(zFP2);
@@ -1333,7 +1388,7 @@ void testset_fp(void){
 
   n = g.szTest/25 + 2;
   speedtest1_begin_test(110, "%d range queries", n);
-  speedtest1_prepare("SELECT sum(b) FROM t1 WHERE a BETWEEN ?1 AND ?2");
+  speedtest1_prepare("SELECT sum(b) FROM z1 WHERE a BETWEEN ?1 AND ?2");
   for(i=1; i<=n; i++){
     speedtest1_random_ascii_fp(zFP1);
     speedtest1_random_ascii_fp(zFP2);
@@ -1345,15 +1400,15 @@ void testset_fp(void){
 
   speedtest1_begin_test(120, "CREATE INDEX three times");
   speedtest1_exec("BEGIN;");
-  speedtest1_exec("CREATE INDEX t1a ON t1(a);");
-  speedtest1_exec("CREATE INDEX t1b ON t1(b);");
-  speedtest1_exec("CREATE INDEX t1ab ON t1(a,b);");
+  speedtest1_exec("CREATE INDEX t1a ON z1(a);");
+  speedtest1_exec("CREATE INDEX t1b ON z1(b);");
+  speedtest1_exec("CREATE INDEX t1ab ON z1(a,b);");
   speedtest1_exec("COMMIT;");
   speedtest1_end_test();
 
   n = g.szTest/3 + 2;
   speedtest1_begin_test(130, "%d indexed range queries", n);
-  speedtest1_prepare("SELECT sum(b) FROM t1 WHERE a BETWEEN ?1 AND ?2");
+  speedtest1_prepare("SELECT sum(b) FROM z1 WHERE a BETWEEN ?1 AND ?2");
   for(i=1; i<=n; i++){
     speedtest1_random_ascii_fp(zFP1);
     speedtest1_random_ascii_fp(zFP2);
@@ -1365,14 +1420,14 @@ void testset_fp(void){
 
   n = g.szTest*5000;
   speedtest1_begin_test(140, "%d calls to round()", n);
-  speedtest1_exec("SELECT sum(round(a,2)+round(b,4)) FROM t1;");
+  speedtest1_exec("SELECT sum(round(a,2)+round(b,4)) FROM z1;");
   speedtest1_end_test();
 
 
   speedtest1_begin_test(150, "%d printf() calls", n*4);
   speedtest1_exec(
     "WITH c(fmt) AS (VALUES('%%g'),('%%e'),('%%!g'),('%%.20f'))"
-    "SELECT sum(printf(fmt,a)) FROM t1, c"
+    "SELECT sum(printf(fmt,a)) FROM z1, c"
   );
   speedtest1_end_test();
 }
@@ -1458,8 +1513,8 @@ void testset_rtree(int p1, int p2){
   speedtest1_end_test();
 
   speedtest1_begin_test(101, "Copy from rtree to a regular table");
-  speedtest1_exec("CREATE TABLE t1(id INTEGER PRIMARY KEY,x0,x1,y0,y1,z0,z1)");
-  speedtest1_exec("INSERT INTO t1 SELECT * FROM rt1");
+  speedtest1_exec("CREATE TABLE z1(id INTEGER PRIMARY KEY,x0,x1,y0,y1,z0,z1)");
+  speedtest1_exec("INSERT INTO z1 SELECT * FROM rt1");
   speedtest1_end_test();
 
   n = g.szTest*200;
@@ -1477,7 +1532,7 @@ void testset_rtree(int p1, int p2){
   if( g.bVerify ){
     n = g.szTest*200;
     speedtest1_begin_test(111, "Verify result from 1-D intersect slice queries");
-    speedtest1_prepare("SELECT count(*) FROM t1 WHERE x0>=?1 AND x1<=?2");
+    speedtest1_prepare("SELECT count(*) FROM z1 WHERE x0>=?1 AND x1<=?2");
     iStep = mxCoord/n;
     for(i=0; i<n; i++){
       sqlite3_bind_int(g.pStmt, 1, i*iStep);
@@ -1506,7 +1561,7 @@ void testset_rtree(int p1, int p2){
   if( g.bVerify ){
     n = g.szTest*200;
     speedtest1_begin_test(121, "Verify result from 1-D overlap slice queries");
-    speedtest1_prepare("SELECT count(*) FROM t1 WHERE y1>=?1 AND y0<=?2");
+    speedtest1_prepare("SELECT count(*) FROM z1 WHERE y1>=?1 AND y0<=?2");
     iStep = mxCoord/n;
     for(i=0; i<n; i++){
       sqlite3_bind_int(g.pStmt, 1, i*iStep);
@@ -1602,7 +1657,7 @@ void testset_rtree(int p1, int p2){
   speedtest1_end_test();
 
   speedtest1_begin_test(170, "Restore deleted entries using INSERT OR IGNORE");
-  speedtest1_exec("INSERT OR IGNORE INTO rt1 SELECT * FROM t1");
+  speedtest1_exec("INSERT OR IGNORE INTO rt1 SELECT * FROM z1");
   speedtest1_end_test();
 }
 #endif /* SQLITE_ENABLE_RTREE */
@@ -1887,11 +1942,11 @@ void testset_trigger(void){
 
   speedtest1_exec(
       "BEGIN;"
-      "CREATE TABLE t1(rowid INTEGER PRIMARY KEY, i INTEGER, t TEXT);"
-      "CREATE TABLE t2(rowid INTEGER PRIMARY KEY, i INTEGER, t TEXT);"
+      "CREATE TABLE z1(rowid INTEGER PRIMARY KEY, i INTEGER, t TEXT);"
+      "CREATE TABLE z2(rowid INTEGER PRIMARY KEY, i INTEGER, t TEXT);"
       "CREATE TABLE t3(rowid INTEGER PRIMARY KEY, i INTEGER, t TEXT);"
-      "CREATE VIEW v1 AS SELECT rowid, i, t FROM t1;"
-      "CREATE VIEW v2 AS SELECT rowid, i, t FROM t2;"
+      "CREATE VIEW v1 AS SELECT rowid, i, t FROM z1;"
+      "CREATE VIEW v2 AS SELECT rowid, i, t FROM z2;"
       "CREATE VIEW v3 AS SELECT rowid, i, t FROM t3;"
   );
   for(jj=1; jj<=3; jj++){
@@ -1905,22 +1960,22 @@ void testset_trigger(void){
     }
   }
   speedtest1_exec(
-      "CREATE INDEX i1 ON t1(t);"
-      "CREATE INDEX i2 ON t2(t);"
+      "CREATE INDEX i1 ON z1(t);"
+      "CREATE INDEX i2 ON z2(t);"
       "CREATE INDEX i3 ON t3(t);"
       "COMMIT;"
   );
 
   speedtest1_begin_test(100, "speed4p-join1");
   speedtest1_prepare(
-      "SELECT * FROM t1, t2, t3 WHERE t1.oid = t2.oid AND t2.oid = t3.oid"
+      "SELECT * FROM z1, z2, t3 WHERE z1.oid = z2.oid AND z2.oid = t3.oid"
   );
   speedtest1_run();
   speedtest1_end_test();
 
   speedtest1_begin_test(110, "speed4p-join2");
   speedtest1_prepare(
-      "SELECT * FROM t1, t2, t3 WHERE t1.t = t2.t AND t2.t = t3.t"
+      "SELECT * FROM z1, z2, t3 WHERE z1.t = z2.t AND z2.t = t3.t"
   );
   speedtest1_run();
   speedtest1_end_test();
@@ -1957,8 +2012,8 @@ void testset_trigger(void){
 
   speedtest1_begin_test(150, "speed4p-subselect1");
   speedtest1_prepare("SELECT "
-      "(SELECT t FROM t1 WHERE rowid = ?1),"
-      "(SELECT t FROM t2 WHERE rowid = ?1),"
+      "(SELECT t FROM z1 WHERE rowid = ?1),"
+      "(SELECT t FROM z2 WHERE rowid = ?1),"
       "(SELECT t FROM t3 WHERE rowid = ?1)"
   );
   for(jj=0; jj<NROW2; jj++){
@@ -1969,7 +2024,7 @@ void testset_trigger(void){
 
   speedtest1_begin_test(160, "speed4p-rowid-update");
   speedtest1_exec("BEGIN");
-  speedtest1_prepare("UPDATE t1 SET i=i+1 WHERE rowid=?1");
+  speedtest1_prepare("UPDATE z1 SET i=i+1 WHERE rowid=?1");
   for(jj=0; jj<NROW2; jj++){
     sqlite3_bind_int(g.pStmt, 1, jj);
     speedtest1_run();
@@ -1979,7 +2034,7 @@ void testset_trigger(void){
 
   speedtest1_exec("CREATE TABLE t5(t TEXT PRIMARY KEY, i INTEGER);");
   speedtest1_begin_test(170, "speed4p-insert-ignore");
-  speedtest1_exec("INSERT OR IGNORE INTO t5 SELECT t, i FROM t1");
+  speedtest1_exec("INSERT OR IGNORE INTO t5 SELECT t, i FROM z1");
   speedtest1_end_test();
 
   speedtest1_exec(
@@ -2145,11 +2200,11 @@ static int xCompileOptions(void *pCtx, int nVal, char **azVal, char **azCol){
   printf("-- Compile option: %s\n", azVal[0]);
   return SQLITE_OK;
 }
-
 int main(int argc, char **argv){
   int doAutovac = 0;            /* True for --autovacuum */
   int cacheSize = 0;            /* Desired cache size.  0 means default */
   int doExclusive = 0;          /* True for --exclusive */
+  int doFullFSync = 0;          /* True for --fullfsync */
   int nHeap = 0, mnHeap = 0;    /* Heap size from --heap */
   int doIncrvac = 0;            /* True for --incrvacuum */
   const char *zJMode = 0;       /* Journal mode */
@@ -2163,7 +2218,10 @@ int main(int argc, char **argv){
   int nThread = 0;              /* --threads value */
   int mmapSize = 0;             /* How big of a memory map to use */
   int memDb = 0;                /* --memdb.  Use an in-memory database */
+  int openFlags = SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE
+    ;                           /* SQLITE_OPEN_xxx flags. */
   char *zTSet = "main";         /* Which --testset torun */
+  const char * zVfs = 0;        /* --vfs NAME */
   int doTrace = 0;              /* True for --trace */
   const char *zEncoding = 0;    /* --utf16be or --utf16le */
   const char *zDbName = 0;      /* Name of the test database */
@@ -2175,10 +2233,19 @@ int main(int argc, char **argv){
   int i;                        /* Loop counter */
   int rc;                       /* API return code */
 
+#ifdef SQLITE_SPEEDTEST1_WASM
+  /* Resetting all state is important for the WASM build, which may
+  ** call main() multiple times. */
+  memset(&g, 0, sizeof(g));
+  iTestNumber = 0;
+#endif
 #ifdef SQLITE_CKSUMVFS_STATIC
   sqlite3_register_cksumvfs(0);
 #endif
-
+  /*
+  ** Confirms that argc has at least N arguments following argv[i]. */
+#define ARGC_VALUE_CHECK(N)                                       \
+  if( i>=argc-(N) ) fatal_error("missing argument on %s\n", argv[i])
   /* Display the version of SQLite being tested */
   printf("-- Speedtest1 for SQLite %s %.48s\n",
          sqlite3_libversion(), sqlite3_sourceid());
@@ -2195,32 +2262,35 @@ int main(int argc, char **argv){
       do{ z++; }while( z[0]=='-' );
       if( strcmp(z,"autovacuum")==0 ){
         doAutovac = 1;
+      }else if( strcmp(z,"big-transactions")==0 ){
+        g.doBigTransactions = 1;
       }else if( strcmp(z,"cachesize")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
-        i++;
-        cacheSize = integerValue(argv[i]);
+        ARGC_VALUE_CHECK(1);
+        cacheSize = integerValue(argv[++i]);
       }else if( strcmp(z,"exclusive")==0 ){
         doExclusive = 1;
+      }else if( strcmp(z,"fullfsync")==0 ){
+        doFullFSync = 1;
       }else if( strcmp(z,"checkpoint")==0 ){
         g.doCheckpoint = 1;
       }else if( strcmp(z,"explain")==0 ){
         g.bSqlOnly = 1;
         g.bExplain = 1;
       }else if( strcmp(z,"heap")==0 ){
-        if( i>=argc-2 ) fatal_error("missing arguments on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(2);
         nHeap = integerValue(argv[i+1]);
         mnHeap = integerValue(argv[i+2]);
         i += 2;
       }else if( strcmp(z,"incrvacuum")==0 ){
         doIncrvac = 1;
       }else if( strcmp(z,"journal")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         zJMode = argv[++i];
       }else if( strcmp(z,"key")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         zKey = argv[++i];
       }else if( strcmp(z,"lookaside")==0 ){
-        if( i>=argc-2 ) fatal_error("missing arguments on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(2);
         nLook = integerValue(argv[i+1]);
         szLook = integerValue(argv[i+2]);
         i += 2;
@@ -2234,9 +2304,11 @@ int main(int argc, char **argv){
 #endif
 #if SQLITE_VERSION_NUMBER>=3007017
       }else if( strcmp(z, "mmap")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         mmapSize = integerValue(argv[++i]);
  #endif
+      }else if( strcmp(z,"nomutex")==0 ){
+        openFlags |= SQLITE_OPEN_NOMUTEX;
       }else if( strcmp(z,"nosync")==0 ){
         noSync = 1;
       }else if( strcmp(z,"notnull")==0 ){
@@ -2246,7 +2318,7 @@ int main(int argc, char **argv){
         fatal_error("The --output option is not supported with"
                     " -DSPEEDTEST_OMIT_HASH\n");
 #else
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         i++;
         if( strcmp(argv[i],"-")==0 ){
           g.hashFile = stdout;
@@ -2258,10 +2330,10 @@ int main(int argc, char **argv){
         }
 #endif
       }else if( strcmp(z,"pagesize")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         pageSize = integerValue(argv[++i]);
       }else if( strcmp(z,"pcache")==0 ){
-        if( i>=argc-2 ) fatal_error("missing arguments on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(2);
         nPCache = integerValue(argv[i+1]);
         szPCache = integerValue(argv[i+2]);
         doPCache = 1;
@@ -2269,9 +2341,8 @@ int main(int argc, char **argv){
       }else if( strcmp(z,"primarykey")==0 ){
         g.zPK = "PRIMARY KEY";
       }else if( strcmp(z,"repeat")==0 ){
-        if( i>=argc-1 ) fatal_error("missing arguments on %s\n", argv[i]);
-        g.nRepeat = integerValue(argv[i+1]);
-        i += 1;
+        ARGC_VALUE_CHECK(1);
+        g.nRepeat = integerValue(argv[++i]);
       }else if( strcmp(z,"reprepare")==0 ){
         g.bReprepare = 1;
 #if SQLITE_VERSION_NUMBER>=3006000
@@ -2280,29 +2351,36 @@ int main(int argc, char **argv){
       }else if( strcmp(z,"singlethread")==0 ){
         sqlite3_config(SQLITE_CONFIG_SINGLETHREAD);
 #endif
+      }else if( strcmp(z,"script")==0 ){
+        ARGC_VALUE_CHECK(1);
+        if( g.pScript ) fclose(g.pScript);
+        g.pScript = fopen(argv[++i], "wb");
+        if( g.pScript==0 ){
+          fatal_error("unable to open output file \"%s\"\n", argv[i]);
+        }
       }else if( strcmp(z,"sqlonly")==0 ){
         g.bSqlOnly = 1;
       }else if( strcmp(z,"shrink-memory")==0 ){
         g.bMemShrink = 1;
       }else if( strcmp(z,"size")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         g.szTest = integerValue(argv[++i]);
       }else if( strcmp(z,"stats")==0 ){
         showStats = 1;
       }else if( strcmp(z,"temp")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         i++;
         if( argv[i][0]<'0' || argv[i][0]>'9' || argv[i][1]!=0 ){
           fatal_error("argument to --temp should be integer between 0 and 9");
         }
         g.eTemp = argv[i][0] - '0';
       }else if( strcmp(z,"testset")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         zTSet = argv[++i];
       }else if( strcmp(z,"trace")==0 ){
         doTrace = 1;
       }else if( strcmp(z,"threads")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         nThread = integerValue(argv[++i]);
       }else if( strcmp(z,"utf16le")==0 ){
         zEncoding = "utf16le";
@@ -2313,12 +2391,31 @@ int main(int argc, char **argv){
 #ifndef SPEEDTEST_OMIT_HASH
         HashInit();
 #endif
+      }else if( strcmp(z,"vfs")==0 ){
+        ARGC_VALUE_CHECK(1);
+        zVfs = argv[++i];
       }else if( strcmp(z,"reserve")==0 ){
-        if( i>=argc-1 ) fatal_error("missing argument on %s\n", argv[i]);
+        ARGC_VALUE_CHECK(1);
         g.nReserve = atoi(argv[++i]);
+      }else if( strcmp(z,"stmtscanstatus")==0 ){
+        g.stmtScanStatus = 1;
       }else if( strcmp(z,"without-rowid")==0 ){
-        g.zWR = "WITHOUT ROWID";
+        if( strstr(g.zWR,"WITHOUT")!=0 ){
+          /* no-op */
+        }else if( strstr(g.zWR,"STRICT")!=0 ){
+          g.zWR = "WITHOUT ROWID,STRICT";
+        }else{
+          g.zWR = "WITHOUT ROWID";
+        }
         g.zPK = "PRIMARY KEY";
+      }else if( strcmp(z,"strict")==0 ){
+        if( strstr(g.zWR,"STRICT")!=0 ){
+          /* no-op */
+        }else if( strstr(g.zWR,"WITHOUT")!=0 ){
+          g.zWR = "WITHOUT ROWID,STRICT";
+        }else{
+          g.zWR = "STRICT";
+        }
       }else if( strcmp(z, "help")==0 || strcmp(z,"?")==0 ){
         printf(zHelp, argv[0]);
         exit(0);
@@ -2333,7 +2430,7 @@ int main(int argc, char **argv){
                   argv[i], argv[0]);
     }
   }
-  if( zDbName!=0 ) unlink(zDbName);
+#undef ARGC_VALUE_CHECK
 #if SQLITE_VERSION_NUMBER>=3006001
   if( nHeap>0 ){
     pHeap = malloc( nHeap );
@@ -2356,8 +2453,23 @@ int main(int argc, char **argv){
 #endif
   sqlite3_initialize();
 
+  if( zDbName!=0 ){
+    sqlite3_vfs *pVfs = sqlite3_vfs_find(zVfs);
+    /* For some VFSes, e.g. opfs, unlink() is not sufficient. Use the
+    ** selected (or default) VFS's xDelete method to delete the
+    ** database. This is specifically important for the "opfs" VFS
+    ** when running from a WASM build of speedtest1, so that the db
+    ** can be cleaned up properly. For historical compatibility, we'll
+    ** also simply unlink(). */
+    if( pVfs!=0 ){
+      pVfs->xDelete(pVfs, zDbName, 1);
+    }
+    unlink(zDbName);
+  }
+
   /* Open the database and the input file */
-  if( sqlite3_open(memDb ? ":memory:" : zDbName, &g.db) ){
+  if( sqlite3_open_v2(memDb ? ":memory:" : zDbName, &g.db,
+                      openFlags, zVfs) ){
     fatal_error("Cannot open database file: %s\n", zDbName);
   }
 #if SQLITE_VERSION_NUMBER>=3006001
@@ -2370,6 +2482,9 @@ int main(int argc, char **argv){
   if( g.nReserve>0 ){
     sqlite3_file_control(g.db, 0, SQLITE_FCNTL_RESERVE_BYTES, &g.nReserve);
   }
+  if( g.stmtScanStatus ){
+    sqlite3_db_config(g.db, SQLITE_DBCONFIG_STMT_SCANSTATUS, 1, 0);
+  }
 
   /* Set database connection options */
   sqlite3_create_function(g.db, "random", 0, SQLITE_UTF8, 0, randomFunc, 0, 0);
@@ -2400,7 +2515,11 @@ int main(int argc, char **argv){
   if( cacheSize ){
     speedtest1_exec("PRAGMA cache_size=%d", cacheSize);
   }
-  if( noSync ) speedtest1_exec("PRAGMA synchronous=OFF");
+  if( noSync ){
+    speedtest1_exec("PRAGMA synchronous=OFF");
+  }else if( doFullFSync ){
+    speedtest1_exec("PRAGMA fullfsync=ON");
+  }
   if( doExclusive ){
     speedtest1_exec("PRAGMA locking_mode=EXCLUSIVE");
   }
@@ -2532,6 +2651,9 @@ int main(int argc, char **argv){
     displayLinuxIoStats(stdout);
   }
 #endif
+  if( g.pScript ){
+    fclose(g.pScript);
+  }
 
   /* Release memory */
   free( pLook );
@@ -2539,3 +2661,13 @@ int main(int argc, char **argv){
   free( pHeap );
   return 0;
 }
+
+#ifdef SQLITE_SPEEDTEST1_WASM
+/*
+** A workaround for some inconsistent behaviour with how
+** main() does (or does not) get exported to WASM.
+*/
+int wasm_main(int argc, char **argv){
+  return main(argc, argv);
+}
+#endif
